<?php

namespace App\Http\Controllers\Admin;

use App\Models\Ptc;
use App\Models\Fund;
use App\Models\Plan;
use App\Models\Language;
use App\Models\Referral;
use App\Models\Configure;
use App\Http\Traits\Notify;
use App\Http\Traits\Upload;
use App\Models\PlanDetails;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use Stevebauman\Purify\Facades\Purify;
use Illuminate\Support\Facades\Validator;

class ManagePlanController extends Controller
{
    use Upload, Notify;

    public function __construct()
    {
        $this->middleware(function ($request, $next) {
            $this->user = Auth::guard('admin')->user();
            return $next($request);
        });
    }


    // Referral
    public function referralCommissionAction(Request $request)
    {
        $configure = Configure::firstOrNew();
        $reqData = Purify::clean($request->except('_token', '_method'));

        $configure->fill($reqData)->save();

        config(['basic.deposit_commission' => (int)$reqData['deposit_commission']]);
        config(['basic.plan_purchase_commission' => (int)$reqData['plan_purchase_commission']]);
        config(['basic.ptc_view_commission' => (int)$reqData['ptc_view_commission']]);
        $fp = fopen(base_path() . '/config/basic.php', 'w');
        fwrite($fp, '<?php return ' . var_export(config('basic'), true) . ';');
        fclose($fp);

        return back()->with('success', 'Update Successfully.');
    }


    public function referralCommission()
    {
        $data['control'] = Configure::firstOrNew();
        $data['referrals'] = Referral::get();
        return view('admin.plan.referral-commission', $data);
    }


    public function referralCommissionStore(Request $request)
    {
        $request->validate([
            'level*' => 'required|integer|min:1',
            'percent*' => 'required|numeric',
            'commission_type' => 'required',
        ]);

        Referral::where('commission_type',$request->commission_type)->delete();

        for ($i = 0; $i < count($request->level); $i++){
            $referral = new Referral();
            $referral->commission_type = $request->commission_type;
            $referral->level = $request->level[$i];
            $referral->percent = $request->percent[$i];
            $referral->save();
        }

        return back()->with('success', 'Level Bonus Has been Updated.');
    }


    // Plan
    public function plan()
    {
        $managePlans = Plan::with('details')->latest()->get();
        return view('admin.plan.planList', compact('managePlans'));
    }

    public function planCreate()
    {
        $languages = Language::all();
        $max_referral_levels = Referral::max('level');
        return view('admin.plan.planCreate', compact('languages', 'max_referral_levels'));
    }


    public function planStore(Request $request, $language)
    {
        $purifiedData = Purify::clean($request->except('_token', '_method'));

        $rules = [
            'name.*' => 'required|max:20',
            'tag.*' => 'max:10',
            'short_title.*' => 'required|max:250',
            'price' => 'required|numeric',
            'validity' => 'required|min:1',
            'daily_ad_limit' => 'required|numeric|min:1',
            'referral_level' => 'required|numeric|min:0',
        ];
        $message = [
            'name.*.required' => 'Plan name field is required',
            'name.*.max' => 'Plan name field may not be greater than :max characters.',
            'tag.*.max' => 'Tag name field may not be greater than :max characters.',
            'short_title.*.required' => 'Short Title field is required',
            'short_title.*.max' => 'Short Title field may not be greater than :max characters.',
            'price.required' => 'Price field is required',
            'price.numeric' => 'Price must be a numeric value',
            'validity.*.required' => 'Validity field is required',
            'validity.*.min' => 'Validity may not be less than :min',
            'daily_ad_limit.*.required' => 'Daily Ad Limit field is required',
            'daily_ad_limit.*.min' => 'Daily Ad Limi may not be less than :min',
            'referral_level.*.required' => 'Referral Level Commission field is required',
            'referral_level.*.min' => 'Referral Commission may not be less than :min level',
        ];

        $validate = Validator::make($purifiedData, $rules, $message);

        if ($validate->fails()) {
            return back()->withInput()->withErrors($validate);
        }

        $plan = new Plan();
        $plan->price = $purifiedData['price'];
        $plan->daily_ad_limit = (int) $purifiedData['daily_ad_limit'];
        $plan->validity = (int) $purifiedData['validity'];
        $plan->referral_level = (int) $purifiedData['referral_level'];
        $plan->status = (int) $purifiedData['status'];

        $plan->save();

        $plan->details()->create([
            'language_id' => $language,
            'name' => $purifiedData["name"][$language],
            'tag' => $purifiedData["tag"][$language],
            'short_title' => $purifiedData["short_title"][$language],
        ]);

        return redirect()->route('admin.planList')->with('success', 'Plan Successfully Saved');
    }


    public function planDelete($id)
    {
        $planPurchasedExist = Fund::where('plan_id', $id)->where('status', 1)->exists();
        if ($planPurchasedExist){
            return back()->with('error', 'Can\'t delete this Plan, because this Plan is already purchased by user');
        }
        else{
            $planData = Plan::findOrFail($id);
            $planData->delete();
            return back()->with('success', 'Plan has been deleted');
        }
    }


    public function planEdit($id)
    {
        $languages = Language::all();
        $max_referral_levels = Referral::max('level');
        $planDetails = PlanDetails::with('plan')->where('plan_id', $id)->get()->groupBy('language_id');
        return view('admin.plan.planEdit', compact('languages', 'planDetails', 'id', 'max_referral_levels'));
    }


    public function planUpdate(Request $request, $id, $language_id)
    {
        $purifiedData = Purify::clean($request->except('image', '_token', '_method'));

        $rules = [
            'name.*' => 'required|max:20',
            'tag.*' => 'max:10',
            'short_title.*' => 'required|max:250',
            'price' => 'sometimes|required|numeric',
            'validity' => 'sometimes|required|min:1',
            'daily_ad_limit' => 'sometimes|required|numeric|min:1',
            'referral_level' => 'sometimes|required|numeric|min:0',
        ];
        $message = [
            'name.*.required' => 'Plan name field is required',
            'name.*.max' => 'Plan name field may not be greater than :max characters.',
            'tag.*.max' => 'Tag name field may not be greater than :max characters.',
            'short_title.*.required' => 'Short Title field is required',
            'short_title.*.max' => 'Short Title field may not be greater than :max characters.',
            'price.required' => 'Price field is required',
            'price.numeric' => 'Price must be a numeric value',
            'validity.*.required' => 'Validity field is required',
            'validity.*.min' => 'Validity may not be less than :min',
            'daily_ad_limit.*.required' => 'Daily Ad Limit field is required',
            'daily_ad_limit.*.min' => 'Daily Ad Limi may not be less than :min',
            'referral_level.*.required' => 'Referral Level Commission field is required',
            'referral_level.*.min' => 'Referral Commission may not be less than :min level',
        ];

        $validate = Validator::make($purifiedData, $rules, $message);

        if ($validate->fails()) {
            return back()->withInput()->withErrors($validate);
        }

        $plan = Plan::findOrFail($id);

        if($request->has('price')){
            $plan->price =$purifiedData['price'];
        }

        if($request->has('validity')){
            $plan->validity =(int) $purifiedData['validity'];
        }

        if($request->has('daily_ad_limit')){
            $plan->daily_ad_limit = (int) $purifiedData['daily_ad_limit'];
        }

        if($request->has('referral_level')){
            $plan->referral_level = (int) $purifiedData['referral_level'];
        }

        if($request->has('status')){
            $plan->status = (int) $purifiedData['status'];
        }

        $plan->save();

        $plan->details()->updateOrCreate([
            'language_id' => $language_id
        ],
            [
                'name' => $purifiedData["name"][$language_id],
                'tag' => $purifiedData["tag"][$language_id],
                'short_title' => $purifiedData["short_title"][$language_id],
            ]
        );

        return back()->with('success', 'Plan Successfully Updated');
    }


    // Ads Settings
    public function advertisementSettings()
    {
        $siteConfigure = Configure::first();
        return view('admin.manage_advertisement.ad_settings', compact('siteConfigure'));
    }


    public function storeAdvertisementSettings(Request $request)
    {
        $siteConfigure = Configure::first();

        $siteConfigure->ads_setting = [
            'ad_price' => $request->ad_price,
            'amount_for_user' => $request->amount_for_user,
        ];
        $siteConfigure->user_ads_status = $request->user_ads_status;
        $siteConfigure->save();

        return redirect()->back()->with('success', 'Advertisements Setting Successfully Updated');
    }


    public function manageAdvertisement()
    {
        $allAds = Ptc::with('user')->orderBy('id', 'desc')->paginate(config('basic.paginate'));
        return view('admin.manage_advertisement.ads_list', compact('allAds'));
    }


    public function createAdvertisement()
    {
        $siteConfigure = Configure::first();
        return view('admin.manage_advertisement.ads_create', compact('siteConfigure'));
    }


    public function storeAdvertisement(Request $request)
    {
        $rules = [
            'ads_name' => 'required|max:30|min:3',
            'user_amount' => 'required|numeric',
            'ads_type' => 'required|in:1,2,3,4|integer',
            'ads_duration' => 'required|min:1|integer|max:10',
            'website_link' => 'nullable|url|required_without_all:banner_image,script,youtube',
            'script' => 'nullable|required_without_all:website_link,banner_image,youtube',
            'youtube' => 'nullable|required_without_all:website_link,banner_image,script',
            'banner_image' => 'nullable|mimes:jpeg,jpg,png,gif|required_without_all:website_link,script,youtube',
            'max_show_limit' => 'required|integer|min:1',
        ];
        $message = [
            'ads_name.required'   => 'Ads Name field is required',
            'ads_name.max'   => 'Ads Name field may not be greater than :max characters.',
            'ads_name.min'   => 'Ads Name field may not be less than :min characters.',
            'user_amount.required'   => 'User Amount field is required',
            'user_amount.numeric'   => 'User Amount field must be a number.',
            'ads_type.required'   => 'Ads Type field is required',
            'ads_duration.required'   => 'Ads Duration field is required',
            'ads_duration.max'   => 'Ads Duration field may not be greater than :max seconds.',
            'ads_duration.min'   => 'Ads Duration field may not be less than :min seconds.',
            'website_link.required_without_all'   => 'The URL/Link field is required.',
            'script.required_without_all'   => 'The Code/Script field is required.',
            'youtube.required_without_all'   => 'The Youtube Embeded Link field is required.',
            'banner_image.required_without_all'   => 'The Picture/Banner field is required.',
            'max_show_limit.required'   => 'Maximum Show Limit field is required',
            'max_show_limit.integer'   => 'Maximum Show Limit field must be a number.',
        ];

        $validate = Validator::make($request->all(), $rules, $message);

        if ($validate->fails()) {
            return back()->withInput()->withErrors($validate);
        }

        $PTC = new Ptc();
        $PTC->user_id = 0;
        $PTC->ads_name = $request->ads_name;
        $PTC->user_amount = $request->user_amount;
        $PTC->ads_type = $request->ads_type;
        $PTC->ads_duration = $request->ads_duration;
        $PTC->max_show_limit = $request->max_show_limit;
        $PTC->remaining = $request->max_show_limit;
        $PTC->status = $request->status;

        if($request->ads_type == 1){
            $PTC->ads_body = $request->website_link;
        }elseif($request->ads_type == 3){
            if ($request->hasFile('banner_image')) {
                $PTC->ads_body = $this->uploadImage($request->banner_image, config('location.advertisement.path'), null, null);
            }
        }elseif($request->ads_type == 2){
            $PTC->ads_body = $request->youtube;
        }elseif($request->ads_type == 4){
            $PTC->ads_body = $request->script;
        }

        $PTC->save();

        return redirect()->route('admin.manage.advertisement')->with('success', 'Advertisements Saved Successfully');
    }


    public function editAdvertisement($id)
    {
        $ad_data = Ptc::findOrFail($id);
        return view('admin.manage_advertisement.ads_edit', compact('ad_data'));
    }


    public function updateAdvertisement(Request $request, $id)
    {
        $rules = [
            'ads_name' => 'required|max:30|min:3',
            'user_amount' => 'required|numeric',
            'ads_duration' => 'required|min:1|integer|max:10',
            'website_link' => 'nullable|url',
            'script' => 'nullable',
            'youtube' => 'nullable',
            'banner_image' => 'nullable|mimes:jpeg,jpg,png,gif',
            'max_show_limit' => 'required|integer|min:1',
        ];
        $message = [
            'ads_name.required'   => 'Ads Name field is required',
            'ads_name.max'   => 'Ads Name field may not be greater than :max characters.',
            'ads_name.min'   => 'Ads Name field may not be less than :min characters.',
            'user_amount.required'   => 'User Amount field is required',
            'user_amount.numeric'   => 'User Amount field must be a number.',
            'ads_duration.required'   => 'Ads Duration field is required',
            'ads_duration.max'   => 'Ads Duration field may not be greater than :max seconds.',
            'ads_duration.min'   => 'Ads Duration field may not be less than :min seconds.',
            'max_show_limit.required'   => 'Maximum Show Limit field is required',
            'max_show_limit.integer'   => 'Maximum Show Limit field must be a number',
        ];

        $validate = Validator::make($request->all(), $rules, $message);

        if ($validate->fails()) {
            return back()->withInput()->withErrors($validate);
        }

        $PTC = Ptc::findOrFail($id);
        $PTC->ads_name = $request->ads_name;
        $PTC->user_amount = $request->user_amount;
        $PTC->ads_type = $request->ads_type;
        $PTC->ads_duration = $request->ads_duration;
        $PTC->max_show_limit = $request->max_show_limit;
        $PTC->status = $request->status;

        if($request->ads_type == 1){
            $PTC->ads_body = $request->website_link;
        }elseif($request->ads_type == 3){
            if ($request->hasFile('banner_image')) {
                $PTC->ads_body = $this->uploadImage($request->banner_image, config('location.advertisement.path'), null, $PTC->ads_body);
            }
        }elseif($request->ads_type == 2){
            $PTC->ads_body = $request->youtube;
        }elseif($request->ads_type == 4){
            $PTC->ads_body = $request->script;
        }

        $PTC->save();

        return redirect()->route('admin.manage.advertisement')->with('success', 'Advertisements Updated Successfully');
    }


    public function deleteAdvertisement($id)
    {
        $advertisement = Ptc::findOrFail($id);

        if($advertisement->ads_type == 3){
            $old_image = $advertisement->ads_body;
            $location  = config('location.advertisement.path');
            if (!empty($old_image)) {
                unlink($location . '/' . $old_image);
            }
        }

        $advertisement->delete();

        return redirect()->back()->with('success', 'Advertisements Deleted Successfully');
    }


    public function searchAdvertisement(Request $request)
    {
        $search = $request->all();

        $allAds = Ptc::orderBy('id', 'DESC')
                ->when(isset($search['ads_name']), function ($query) use ($search) {
                    return $query->where('ads_name', 'LIKE', '%' . $search['ads_name'] . '%');
                })
                ->when(isset($search['ads_type']), function ($query) use ($search) {
                    return $query->where('ads_type', $search['ads_type']);
                })
                ->when(isset($search['status']), function ($query) use ($search) {
                    return $query->where('status', $search['status']);
                })
                ->paginate(config('basic.paginate'));

        $allAds->appends($search);

        return view('admin.manage_advertisement.ads_list', compact('allAds'));
    }


}
